# -*- coding: utf-8 -*-
## PlonePopoll: A Plone poll tool
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Installation module for PlonePopoll
"""
__docformat__ = 'restructuredtext'

# Python imports
from StringIO import StringIO

# CMF imports
from Products.CMFCore.utils import getToolByName

# AT imports
from Products.Archetypes import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin

# Product imports
from Products.PlonePopoll.PlonePopollTool import PlonePopollTool
from Products.PlonePopoll.config import *

tool = 'PlonePopoll Tool'
import sys

def setupCustomSlots(self, out):
   """ sets up the custom slots"""
   #use the right zpt

   popoll_slot = 'here/popoll_slot/macros/popollBox'
   popoll_slot2 = 'here/popoll_slot2/macros/popollBox'
   popoll_portlet = 'here/portlet_popoll/macros/portlet'
   if hasattr(self, "right_slots"):
       right_slots_list=list(self.right_slots)
       if  popoll_slot in right_slots_list:
            self._delProperty('right_slots')
            right_slots_list.remove(popoll_slot)
            self._setProperty('right_slots',tuple(right_slots_list),'lines')
            out.write("Old right slot '%s' has been removed" % popoll_slot)
       elif popoll_slot2 in right_slots_list:
            self._delProperty('right_slots')
            right_slots_list.remove(popoll_slot2)
            self._setProperty('right_slots',tuple(right_slots_list),'lines')
            out.write("Old right slot '%s' has been removed\n" % popoll_slot2)
       if popoll_portlet in right_slots_list:
          out.write("Right slot for popoll exists already\n")
       else:
          self._delProperty('right_slots')
          right_slots_list.append(popoll_portlet)
          self._setProperty('right_slots',tuple(right_slots_list),'lines')
          out.write("Added right slot for popoll\n")



def setupTools(self, out):
    addTool = self.manage_addProduct[PROJECTNAME].manage_addTool
    found = 0
    for obj in self.objectValues():
        if obj.meta_type == tool:
            found = 1
            
    if found:
        PlonePopollTool = getToolByName(self, TOOL_ID)
        backend = PlonePopollTool.getBackend()
        doMigration(self, backend)
        out.write("Migration to PlonePoll -multiple choice edition- successful.\n")

    if not found:
        addTool(tool, None)
        self.portal_popoll.manage_addProduct[PROJECTNAME].manage_addPlonePopollZODBBackend()
        out.write("Added '%s' tool.\n" % (tool,))
    
    
def install(self):
    out = StringIO()

    sys.modules['Products.PlonePopoll.PlonePopoll'] = sys.modules['Products.PlonePopoll.content.PlonePopoll']

    # install types
    typeInfo = listTypes(PROJECTNAME)
    installTypes(self, out, typeInfo, PROJECTNAME)

    # install skin
    install_subskin(self, out, GLOBALS)
    
    # Install portlet
    setupCustomSlots(self, out)

    # Add portal types to use portal factory
    pftool = getToolByName(self, 'portal_factory')
    pftool.manage_setPortalFactoryTypes(listOfTypeIds = (PROJECTNAME,))
    
    # Install tool
    setupTools(self, out)
    
    install_configlet(self, out)

    provider = getToolByName(self, 'portal_selenium', None)
    if provider:
        # Functional Tests
        action = {'id':PROJECTNAME.lower(),
                  'name':PROJECTNAME,
                  'action':'string:here/get_%s_ftests'%PROJECTNAME.lower(),
                  'condition': '',
                  'permission': 'View',
                  'category':'ftests',
                  'visible': 1}
        provider.addAction(**action)

    typesTool = getToolByName(self, 'portal_types')
    # Set the human readable title explicitly
    t = getattr(typesTool, 'PlonePopoll', None)
    if t:
        t.title = 'Poll'
    
    #raise "Installation OK"
    out.write('Installation completed.\n')
    return out.getvalue()

def install_configlet(self, out):
    try:
        portal_conf=getToolByName(self,'portal_controlpanel')
    except AttributeError:
        print >>out, "Configlet could not be installed"
        return
    try:
        portal_conf.registerConfiglet(**prefs_plonepopoll_configlet)
    except KeyError:
        pass # Get KeyError when registering duplicate configlet.

def uninstall(self):
    out = StringIO()

    # remove the configlet from the portal control panel
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        configTool.unregisterConfiglet(CONFIGLET_ID)
        out.write('Removed PlonePopoll configlet\n')

    uninstall_tool(self, out)
    
    print >> out, "Successfully uninstalled %s." % PROJECTNAME
    return out.getvalue()

def uninstall_tool(self, out):
    try:
        self.manage_delObjects([TOOLNAME])
    except:
        pass
    else:
        print >>out, "PlonePopoll tool removed" 
    
def doMigration(self, backend):
    """
   #Migrate Results of PlonePopoll 2.0x or lower to 2.1
   #In version 2.0x or lower : choice of user was stored as Integer
   #In version 2.1 : choice of user is stored as a list of choice
   #Get all results in PlonePopollBackend and change the type of stored choice
    """
    
    
    msg = ''
    storedResults = backend._getResults()

    #add if non existant
    if not hasattr(backend.aq_explicit, '_internal_version'):
        backend.aq_explicit._internal_version = 0
    #do changes for 2.0x to 2.1
    if backend.aq_explicit._internal_version < 1:
        backend.aq_explicit._internal_version = 1
        for poll_id in storedResults:
             for user in storedResults[poll_id]:
                choices = storedResults[poll_id][user]
                if not type(choices) == type([]):
                     storedResults[poll_id][user] = [choices]
        backend._results = storedResults
        msg = msg + '\nInternal version set to 1.'
    msg = msg + '\nMigration done.'
    return msg

