# -*- coding: utf-8 -*-
## PlonePopoll: A Plone poll tool
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""PlonePopollTool main class"""

from Log import *
from string import find
from Products.CMFCore.utils import UniqueObject, _getAuthenticatedUser, _checkPermission
from Products.CMFCore.utils import getToolByName, _dtmldir
from OFS.SimpleItem import SimpleItem
from OFS import ObjectManager
from Globals import InitializeClass,  MessageDialog, ImageFile, \
     PersistentMapping
from Acquisition import aq_base
from AccessControl.User import nobody
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import *
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

import sys


class PlonePopollTool(UniqueObject, ObjectManager.ObjectManager, SimpleItem, ActionProviderBase):
    """
    PlonePopollTool tool
    """

    id = 'portal_popoll'
    meta_type = 'PlonePopoll Tool'
    _actions = []
    security = ClassSecurityInfo()

    manage_options = (
        { 'label': 'Content',
         'action': 'manage_main',},
        { 'label': 'Migration',
          'action': 'manage_migration' },
        ) + ActionProviderBase.manage_options + SimpleItem.manage_options

    #
    #   ZMI methods
    #
    #security.declareProtected(ManagePortal, 'manage_migration' )
    manage_migration = PageTemplateFile('www/manage_migration', globals())
    


    def _post_init(self, backend_constructor_id):
        """
        _post_init(self, backend_constructor_id) => called from manage_add method, acquired within ZODB (__init__ is not)
        """
        self.portlet_configuration = 'newest'
        # Instanciate and initialize backend
        eval("""self.manage_addProduct['PlonePopollTool'].manage_add%s()""" % backend_constructor_id)

    #                                                                           #
    #                   PlonePopollToolBackend management                       #
    #                                                                           #


    def listBackends(self, ):
        """
        listBackends(self, ) => return a list of available backends
        """
        pass            # $$$ TODO !


    def getBackend(self,):
        """
        getBackend(self,) => return the backend
        """
        ret = self.PlonePopollBackend                      #       $$$ UNSAFE ! HAVE TO FIND A BETTER WAY OF RETURNING A SECURITY-WRAPPED OBJECT
        Log(LOG_DEBUG, "getBackend", ret)
        return ret

    #                                                                           #
    #                           Voting interface                                #
    #                                                                           #
    _unicity = 0

    def getVoteUnicity(self, poll_id, create = 1):
        """
        getVoteUnicity(self, poll_id, create = 1) => string or int or None

        return a tag that allows PlonePopollTool to think that this vote isn't
        made twice for the same user.

        We first try to use the username as a tag. If it's not available or if
        the user is anonymous, we use a counter.

        - poll_id is the poll id (!), ie. a sitewide unique id identifying the poll.
          IT MUST NOT CHANGE DURING THE POLL'S LIFE.
          
        - create is an additional parameter set to 1 by default. If it is true,
          a new unicity token will be automatically generated. If it is false, this
          method returns the FORMER one if it exists, OR None if no token exists.
          This is convenient to check if a user has voted without consuming a
          ZODB transaction. => NOT USED
        """
        # Try to fetch the user name
        unicity = None
        mtool = getToolByName(self, 'portal_membership')
        try:
            unicity = mtool.getAuthenticatedMember().getUserName()
            if mtool.isAnonymousUser():
                unicity = None
            #unicity = self.REQUEST.AUTHENTICATED_USER.getUserName()
            #if unicity == "Anonymous User":
            #    unicity = None
        except:
            raise NameError, 'unicity' + unicity
            Log(LOG_NOTICE, "Unable to find username", sys.exc_info()[0], sys.exc_info()[1])
            pass

        # If we didn't find a valid user name, then we use a counter and store it 
        # in a cookie
        Log(LOG_DEBUG, "Computing unicity")
        if not unicity:
            if self.REQUEST.has_key('AnonymousVoted%s' % (poll_id)):
                unicity = self.REQUEST['AnonymousVoted%s' % (poll_id)]
            else:
                self._unicity += 1
                unicity = self._unicity
                expires = 'Wed, 19 Feb 2020 14:28:00 GMT'
                self.REQUEST.RESPONSE.setCookie(
                    'AnonymousVoted%s' % (poll_id,), str(unicity), path='/', expires=expires,
                    )

        # Add the poll id to the unicity factor
        Log(LOG_DEBUG, "bloub", self._unicity, unicity)
        if unicity:
            unicity = "%s%s" % (unicity, poll_id, )
                
        # Return unicity factor
        return unicity

    #                                                                           #
    #                           PlonePopoll configuration                       #
    #                                                                           #

    def setPortletConfiguration(self, portlet_configuration):
        """
        Configure PlonePopoll portlet
        """
        
        self.portlet_configuration = portlet_configuration
        
    def getPortletConfiguration(self):
        """
        Get configuration of PlonePopoll portlet
        """
        
        return getattr(self, 'portlet_configuration', 'newest')
    
    security.declareProtected(ManagePortal, 'manage_migrate')
    def manage_migrate(self, REQUEST=None):
        """Run Extensions.Migration.migrateToArchetypes.
        """
        from Products.PlonePopoll.Extensions import Migration
        out = Migration.migrateToArchetypes(self, REQUEST)
        return out
    
        

InitializeClass(PlonePopollTool)



#                                                                           #
#                           PlonePopollToolBackend management               #
#                                                                           #

def registerBackend(self, *args, **kw):
    """
    registerBackend(self, ...) => register a backend class into the system
    """
    pass        # $$$ TODO !

