# -*- coding: utf-8 -*-
## PlonePopoll: A Plone poll tool
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
PlonePopoll main class
"""

from zLOG import LOG, DEBUG, INFO


from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
import string

# CMF imports
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.PortalContent import PortalContent


# Archetypes imports
from Products.Archetypes.public import *
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *


from Products.PlonePopoll.config import PROJECTNAME, I18N_DOMAIN
from Products.PlonePopoll.content.schemata import PollSchema
from Products.PlonePopoll.config import *
from Products.generator import i18n

from AccessControl import getSecurityManager, ClassSecurityInfo, Unauthorized
import OFS
from OFS import Folder


import sys
import os.path
import types
import time, random, md5, socket

try:
    _v_network = socket.gethostbyname(socket.gethostname())
except:
    _v_network = random.random() * 100000000000000000L


class PlonePopoll(BaseContent):
    """PlonePopoll class"""

    id             = 'PlonePopoll'
    portal_type    = 'PlonePopoll'    
    meta_type      = 'PlonePopoll'
    archetype_name = 'Poll'    
    description    = 'A Poll'
    content_icon   = 'plonepopoll.gif'  # Icon has to be acquired from a skin folder
    
    default_view   = 'plonepopoll_view'
    immediate_view = 'plonepopoll_view'
    
    
    global_allow        = 1
    allow_discussion    = 1
    _check_multi        = 1
    schema = PollSchema

    
    
    actions = (
        {
        'id'           : 'view',
        'name'         : 'View',
        'action'       : 'string: ${object_url}/plonepopoll_view',
        'permissions'  : (CMFCorePermissions.View, ),
        'category'      : 'object',
        },
        {
        'id'           : 'results',
        'name'         : 'Results',
        'action'       : 'string: ${object_url}/plonepopoll_results',
        'permissions'   : (CMFCorePermissions.View, ),
        'category'      : 'object',
        }
    )


    text = ""           # Default text
    
    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)            # $$$ Is this clever ? Isn't it better to make the object private ?
                                                                        # This method makes all class properties & methods protected by View by default

    # Init method
    security.declarePrivate('__init__')
    def __init__(self, id):
        """__init__(self, id)"""
        #PlonePopoll.inheritedAttribute('__init__')(self)
        self.id = id
        self.uid = self.make_uuid(id)


    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        PortalContent.manage_afterClone(self, item)
        self.uid = self.make_uuid(id)

    def make_uuid(self, *args):
        """Generate unique id"""
        t = long(time.time() * 1000)
        r = long(random.random()*100000000000000000L)
        data = str(t)+' '+str(r)+' '+str(_v_network)+' '+str(args)
        data = md5.md5(data).hexdigest()
        return data
    
    
    
    security.declarePrivate('_post_init')
    def _post_init(self):
        """
        _post_init(self) => Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """
        self.indexObject()

    security.declareProtected(CMFCorePermissions.View, 'Description')
    def Description(self):
        "Returns the description of the poll, which is in fact the question"
        return self.getQuestion()

    security.declareProtected(CMFCorePermissions.View, 'getVoteId')
    def getVoteId(self):
        """ return unique vote id """
        if hasattr(self, 'uid'):
            return self.uid
        else:
            return self.getId()
        
    security.declareProtected(PlonePopoll_editPermission, "setPollOptions")
    def setPollOptions(self, question = '', choices = [], choices_number = 1, check_multi = 1):
        """
        setPollOptions(self, question = '', choices = [], choices_number = 1, check_multi = 0)
        - set fields question, choices and max number of choices
        """
        self.getField('question').set(self, question)
        self.getField('number_of_choices').set(self, choices_number)
        self._check_multi = check_multi
        if choices:
            _choices = []
            for c in choices:                                           # We ensure that choices is a list
                _choices.append(c)
            self.getField('choices').set(self, _choices)                      # Ensure persistancy

    #############################################################################
    #                                                                           #
    #                             POLL PROPERTIES                               #
    #                                                                           #
    #############################################################################

    security.declareProtected(PlonePopoll_votePermission, "vote")
    def getPollNumber(self):
        """ compatibility layer """
        return self.getNumber_of_choices()
    
    security.declareProtected(PlonePopoll_votePermission, "vote")
    def vote(self, choices = [], clear = False ):
        """
        vote(self, choices = [], clear = False )
        - calls the backend to register votes
        - Include all the necessary code to pseudo-guarantee the unicity of the vote.
        """
        if not self.isEnabled():
            raise RuntimeError, "This poll is not active."      
        if self.hasVoted():
			self.removeUserVote()
            
        request = self.REQUEST
        response = request.RESPONSE

        checkedCount = len(choices)
        max_votes = int(self.getNumber_of_choices())
        url = self.default_view
        if clear:
            self.clearResults()
            msgstr = "Poll results have been cleared."
            msgid = msgstr
        else:
            if checkedCount > max_votes and self._check_multi:
                msgstr = "You have made ${checked} choices. The maximum authorized is ${max}."
                msgid = 'message_check_count'
            else:
                for choice in choices:
                    # Check that choice is valid
                    if choice >= len(self.choices) and choice >= 0:
                        raise ValueError, "Invalid choice"
                    # Ensure unicity of the vote
                    unicity = getToolByName(self, 'portal_popoll').getVoteUnicity(self.getVoteId(), create=1)
                    LOG("Unicity", INFO, unicity)
                    # Call the method in the backend to store the vote
                    getToolByName(self, 'portal_popoll').getBackend().vote(self.getVoteId(), choice, unicity)
                msgstr = "Vote has been saved."
                msgid = msgstr
                url = 'plonepopoll_results'
        
        message = i18n.translate(I18N_DOMAIN, 
            msgid,
            context = self,
            default = msgstr,
            mapping={'checked': checkedCount, 'max': max_votes}
        )
        response.redirect(
            '%s/%s?portal_status_message=%s' % (self.absolute_url(), url, message)
        )
        


    security.declareProtected(PlonePopoll_votePermission, "hasVoted")
    def hasVoted(self,):
        """
        hasVoted(self,)
        - Return 1 if the user has already voted.
        This is based on the unicity token
        """
        unicity = getToolByName(self, 'portal_popoll').getVoteUnicity(self.getVoteId(), create = 0)
        if unicity:
            return getToolByName(self, 'portal_popoll').getBackend().hasAlreadyVoted(self.getVoteId(), unicity)
        return None

    #############################################################################
    #                                                                           #
    #                           RESULTS MANAGEMENT                              #
    #                                                                           #
    #############################################################################


    security.declareProtected(CMFCorePermissions.View, "getResults")
    def getResults(self, sort = 0):
        """
        getResults(self,)
        -> return results as a TUPLE of tuples (id, count, percentage).

        percentage is 0 <= percentage <= 100
        
        The order is the same as for listChoiceIds if sort if false.
        If sort is true, results are sorted by score (best first)
        """
        res = getToolByName(self, 'portal_popoll').getBackend().getResults(self.getVoteId())
        ret = []
        votes_count = 0
        idx = 0
        for id in self.choices[:]:
            # If a choice has not been selected, it is not in res!
            if not res.has_key(idx):
                ret.append((id, 0))
            else:
                votes_count += res[idx]['count']
                ret.append((id, res[idx]['count']))
            idx += 1

        # Create a new tuple that includes the percentage as well
        result=[]
        for r in ret:
           try:
              percentage=float(r[1])/float(votes_count)*100.0
           except:
              percentage=0.0
           #result.append((r[0],r[1],int(percentage)))  ## We should not do the int(). Fixed (KA & MR) !
           result.append((r[0], r[1], percentage))  

        if sort:
            result.sort(lambda x,y: cmp(y[1], x[1]))

        LOG("getResults for the popoll", DEBUG, result)
        return tuple(result)

    security.declareProtected(CMFCorePermissions.View, "getVotesCount")
    def getVotesCount(self,):
        """
            return total number of votes for this poll
        """
        ret = 0
        for id, count, percentage in self.getResults():
            ret += count

        return ret

    security.declareProtected(CMFCorePermissions.View, "getPersonVoteCount")
    def getPersonVoteCount(self, ):
        """ return number of votes for current user """        
        return getToolByName(self, 'portal_popoll').getBackend().getPersonVoteCount(self.getVoteId())

    security.declareProtected(PlonePopoll_editPermission, "clearResults")
    def clearResults(self, ):
        """ Remove all votes for this poll """
        getToolByName(self, 'portal_popoll').getBackend().clearResults(self.getVoteId())
        
    
    security.declareProtected(CMFCorePermissions.View, "getUnicity")
    def getUnicity(self, ):
        """ """        
        return getToolByName(self, 'portal_popoll').getVoteUnicity(self.getVoteId(), create=0)
    
    security.declareProtected(PlonePopoll_votePermission, "removeUserVote")
    def removeUserVote(self, ):
        """ """        
        unicity = getToolByName(self, 'portal_popoll').getVoteUnicity(self.getVoteId(), create=0)
        getToolByName(self, 'portal_popoll').getBackend().removeUserVote(self.getVoteId(), unicity)
    
    
    #                                                                           #
    #                             CMF CATALOG SUPPORT                           #
    #                                                                           #

    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        "Returns a concatination of all searchable text"
        # Should be overriden by portal objects
        return "%s %s %s" % (self.Title(), self.Description(), self.text)

# Class instanciation
#InitializeClass(PlonePopoll)
registerType(PlonePopoll, PROJECTNAME)
